% script file room1.m
% temperature distribution in a room
% calculation done with Gradient method
% WITHOUT explicit construction of the
% finite difference matrix
% iterate directly with 2D arrays
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all;

% domain: a square [a,b]^2
a = 0; b = 5;

% number of grid points 
% (numbering 1...n INCLUDING the boundary points)
n = 41;


% temperature at the heaters
heaterA = 30;
heaterB = 40;
% temperature at the window
window = 15;

% generate the domain
x = linspace(a,b,n)';
y = linspace(a,b,n)';

% step size 
h = x(2)-x(1);

% generate the right hand side
[X,Y] = ndgrid(x,y);

% rhs is now a 2D array
% with zeros inside and 
% Dirichlet boundary values at the boundary
F = zeros(n,n);

% assign value 20 everywhere on the boundary
F(1,:) = 20;
F(:,1) = 20;
F(n,:) = 20;
F(:,n) = 20;

% overwrite the different values
window_index = find((x<=4.5)&(x>=1.5));
F(window_index,1) = window;

heaterA_index = find((y<=4.5)&(y>=3));
F(n,heaterA_index) = heaterA;

% heater B consists of two parts
heaterB_index1 = find((x<=0.5)&(x>=0));
F(heaterB_index1,n) = heaterB;

heaterB_index2 = find((y<=5)&(y>=4));
F(1,heaterB_index2) = heaterB;



% Gradient method to solve the system
% AU = F
% NOTE: this time work with 2D arrays!!!

maxit = 10;
toler = 1e-1;

U = zeros(n,n); % initialise

% norm of the RHS for computing relative residual
normF = sqrt(sum(sum(F.^2)));

for ctr=1:maxit
  % compute the residual
  % r=F-A*U
  r = F - ApplyLaplace(U,h,n);
  
  % compute 
  % r'*r = r1^2 + r2^2 + ... + rN^2 : sum of all elements squared
  r2 = sum(sum(r.^2)); % attention: may not use 2-norm of a matrix!!!!
  
  % compute r'*A*r
  Ar = ApplyLaplace(r,h,n);
  rAr = sum(sum(r.*Ar));
  
  alpha = r2/rAr;
  
  
  relres = sqrt(r2)/normF; % norm of relative residual
  
  if 0
    % visualise
    figure(1)
    subplot(2,1,1);
    contourf(X,Y,U); colorbar;
    subplot(2,1,2);
    contourf(X,Y,abs(r),40); colorbar;
    drawnow
  end
  
  
  
  if relres<toler
    break; 
  end
    
  % update
  U = U + alpha*r;  
end
  

if ctr==maxit
  warning('Max number of iterations reached!');
end

if 1
  % visualise
  figure(2)
  subplot(2,1,1);
  contourf(X,Y,U,40); colorbar;
  shading flat
  subplot(2,1,2);
  contourf(X,Y,abs(r)); colorbar;
end


return


